<?php

namespace Drupal\eca\Plugin\Action;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Action\ConfigurableActionBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\eca\EcaActionInterface;
use Drupal\eca\Event\FormBase;

/**
 * Disable a form field.
 *
 * @Action(
 *   id = "eca_form_disable_field",
 *   label = @Translation("Disable form field")
 * )
 */
class FormDisableField extends ConfigurableActionBase implements EcaActionInterface{

  /**
   * @var \Symfony\Contracts\EventDispatcher\Event|\Symfony\Component\EventDispatcher\Event
   */
  protected $event;

  /**
   * {@inheritdoc}
   */
  public function execute(): void {
    if ($this->event instanceof FormBase) {
      $form = $this->event->getForm();
      $form[$this->configuration['field_name']]['#disabled'] = TRUE;
      $this->event->setForm($form);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function setEvent($event): EcaActionInterface {
    $this->event = $event;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'field_name' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form['field_name'] = [
      '#type' => 'text',
      '#title' => t('Field name'),
      '#default_value' => $this->configuration['field_name'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['field_name'] = $form_state->getValue('field_name');
  }

  /**
   * {@inheritdoc}
   */
  public function access($object, AccountInterface $account = NULL, $return_as_object = FALSE) {
    $result = AccessResult::allowed();
    return $return_as_object ? $result : $result->isAllowed();
  }

}
