<?php

namespace Drupal\eca\Plugin\Action;

use Drupal;
use Drupal\Core\Action\ConfigurableActionBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * Load referenced entity into token environment.
 *
 * @Action(
 *   id = "eca_load_entity_ref",
 *   label = @Translation("Load referenced entity into token"),
 *   type = "entity"
 * )
 */
class LoadEntityReference extends ConfigurableActionBase {

  /**
   * {@inheritdoc}
   */
  public function execute($entity = NULL): void {
    if ($entity === NULL) {
      return;
    }
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    if (!$entity->hasField($this->configuration['field_entity_reference'])) {
      // TODO: Log this error.
      return;
    }
    if ($referencedEntity = $entity->get($this->configuration['field_entity_reference'])->first()->get('entity')->getTarget()->getValue()) {
      Drupal::service('eca.service.token')->addTokenData($referencedEntity->getEntityTypeId(), $referencedEntity);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'field_entity_reference' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form['field_entity_reference'] = [
      '#type' => 'text',
      '#title' => t('Field name'),
      '#default_value' => $this->configuration['field_entity_reference'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['field_entity_reference'] = $form_state->getValue('field_entity_reference');
  }

  /**
   * {@inheritdoc}
   */
  public function access($object, AccountInterface $account = NULL, $return_as_object = FALSE) {
    /** @var \Drupal\Core\Entity\ContentEntityInterface $object */
    $result = $object->access('view', $account, TRUE);
    return $return_as_object ? $result : $result->isAllowed();
  }

}
