<?php

namespace Drupal\eca\Plugin\Action;

use Drupal\Core\Action\ConfigurableActionBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * Set the value of a referenced entity's field.
 *
 * @Action(
 *   id = "eca_set_field_value_entity_reference",
 *   label = @Translation("Set the value of a referenced entity's field"),
 *   type = "entity"
 * )
 */
class SetFieldValueEntityReference extends ConfigurableActionBase {

  /**
   * {@inheritdoc}
   */
  public function execute($entity = NULL): void {
    if ($entity === NULL) {
      return;
    }
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    if (!$entity->hasField($this->configuration['field_entity_reference'])) {
      // TODO: Log this error.
      return;
    }
    /** @var \Drupal\Core\Entity\ContentEntityInterface $referencedEntity */
    if ($referencedEntity = $entity->get($this->configuration['field_entity_reference'])->first()->get('entity')->getTarget()->getValue()) {
      if (!$referencedEntity->hasField($this->configuration['field_name'])) {
        // TODO: Log this error.
        return;
      }
      $referencedEntity
        ->set($this->configuration['field_name'], $this->configuration['field_value'])
        ->save();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'field_entity_reference' => '',
      'field_name' => '',
      'field_value' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form['field_entity_reference'] = [
      '#type' => 'text',
      '#title' => t('Field name for entity reference'),
      '#default_value' => $this->configuration['field_entity_reference'],
    ];
    $form['field_name'] = [
      '#type' => 'text',
      '#title' => t('Field name'),
      '#default_value' => $this->configuration['field_name'],
    ];
    $form['field_value'] = [
      '#type' => 'text',
      '#title' => t('Field value'),
      '#default_value' => $this->configuration['field_value'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['field_entity_reference'] = $form_state->getValue('field_entity_reference');
    $this->configuration['field_name'] = $form_state->getValue('field_name');
    $this->configuration['field_value'] = $form_state->getValue('field_value');
  }

  /**
   * {@inheritdoc}
   */
  public function access($object, AccountInterface $account = NULL, $return_as_object = FALSE) {
    /** @var \Drupal\Core\Entity\ContentEntityInterface $object */
    $result = $object->access('update', $account, TRUE);
    return $return_as_object ? $result : $result->isAllowed();
  }

}
